@@ -65,6 +65,31 @@
 
 static const char *s_internalMimeType = "application/x-kmenuedit-internal";
 
+class SeparatorWidget : public QWidget
+{
+public:
+    SeparatorWidget()
+        : QWidget(0)
+    {
+    }
+
+protected:
+    void paintEvent(QPaintEvent * /*event*/)
+    {
+        QPainter p(this);
+        // Draw Separator
+        int h = (height() / 2) -1;
+//        if (isSelected()) {
+//            p->setPen( palette().color( QPalette::HighlightedText ) );
+//        } else {
+//            p->setPen( palette().color( QPalette::Text ) );
+//        }
+
+        p.drawLine(2, h, width() - 4, h);
+    }
+};
+
+
 TreeItem::TreeItem(QTreeWidgetItem *parent, QTreeWidgetItem *after, const QString& menuId, bool _m_init)
     : QTreeWidgetItem(parent, after),
       m_hidden(false),
@@ -223,17 +248,18 @@ TreeView::TreeView( KActionCollection *ac, QWidget *parent, const char *name )
     setAcceptDrops(true);
     setMinimumWidth(240);
 
-    setHeaderLabels(QStringList() << QString(""));
+    setHeaderLabels(QStringList() << QLatin1String(""));
     header()->hide();
 
     // listen for creation
-    connect(m_ac->action(NEW_ITEM_ACTION_NAME), SIGNAL(triggered()), SLOT(newitem()));
-    connect(m_ac->action(NEW_SUBMENU_ACTION_NAME), SIGNAL(triggered()), SLOT(newsubmenu()));
+    connect(m_ac->action(NEW_ITEM_ACTION_NAME), &QAction::triggered, this, &TreeView::newitem);
+    connect(m_ac->action(NEW_SUBMENU_ACTION_NAME), &QAction::triggered, this, &TreeView::newsubmenu);
+    connect(m_ac->action(NEW_SEPARATOR_ACTION_NAME), &QAction::triggered, this, &TreeView::newsep);
 
     // listen for copy
-    connect(m_ac->action(CUT_ACTION_NAME), SIGNAL(triggered()), SLOT(cut()));
+    connect(m_ac->action(CUT_ACTION_NAME), &QAction::triggered, this, &TreeView::cut);
     connect(m_ac->action(COPY_ACTION_NAME), SIGNAL(triggered()), SLOT(copy()));
-    connect(m_ac->action(PASTE_ACTION_NAME), SIGNAL(triggered()), SLOT(paste()));
+    connect(m_ac->action(PASTE_ACTION_NAME), &QAction::triggered, this, &TreeView::paste);
 
     // listen for deleting
     connect(m_ac->action(DELETE_ACTION_NAME), SIGNAL(triggered()), SLOT(del()));
@@ -255,21 +281,23 @@ TreeView::TreeView( KActionCollection *ac, QWidget *parent, const char *name )
     connect(m_sortSignalMapper, SIGNAL(mapped(const int)), this, SLOT(sort(const int)));
 
     // connect moving up/down actions
-    connect(m_ac->action(MOVE_UP_ACTION_NAME), SIGNAL(triggered()), SLOT(moveUpItem()));
-    connect(m_ac->action(MOVE_DOWN_ACTION_NAME), SIGNAL(triggered()), SLOT(moveDownItem()));
+    connect(m_ac->action(MOVE_UP_ACTION_NAME), &QAction::triggered, this, &TreeView::moveUpItem);
+    connect(m_ac->action(MOVE_DOWN_ACTION_NAME), &QAction::triggered, this, &TreeView::moveDownItem);
 
     // listen for selection
-    connect(this, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
-            SLOT(itemSelected(QTreeWidgetItem*)));
+    connect(this, &QTreeWidget::currentItemChanged,
+            this, &TreeView::itemSelected);
 
-    m_menuFile = new MenuFile(KStandardDirs::locateLocal("xdgconf-menu", "applications-kmenuedit.menu"));
+    m_menuFile = new MenuFile(KStandardDirs::locateLocal("xdgconf-menu", QStringLiteral("applications-kmenuedit.menu")));
     m_rootFolder = new MenuFolderInfo;
+    m_separator = new MenuSeparatorInfo;
 }
 
 TreeView::~TreeView()
 {
     cleanupClipboard();
     delete m_rootFolder;
+    delete m_separator;
 }
 
 void TreeView::setViewMode(bool showHidden)
@@ -281,6 +309,7 @@ void TreeView::setViewMode(bool showHidden)
     // creation
     m_popupMenu->addAction(m_ac->action(NEW_ITEM_ACTION_NAME));
     m_popupMenu->addAction(m_ac->action(NEW_SUBMENU_ACTION_NAME));
+    m_popupMenu->addAction(m_ac->action(NEW_SEPARATOR_ACTION_NAME));
     m_popupMenu->addSeparator();
 
     // copy
@@ -345,6 +374,10 @@ void TreeView::readMenuFolderInfo(MenuFolderInfo *folderInfo, KServiceGroup::Ptr
             const KService::Ptr service(static_cast<KService*>(e.data()));
             folderInfo->add(new MenuEntryInfo(service), true);
         }
+        else if (e->isType(KST_KServiceSeparator))
+        {
+            folderInfo->add(m_separator, true);
+        }
     }
 }
 
@@ -361,7 +394,7 @@ QString TreeView::findName(KDesktopFile *df, bool deleted)
     QString name = df->readName();
     if (deleted)
     {
-       if (name == "empty")
+       if (name == QLatin1String("empty"))
           name.clear();
        if (name.isEmpty())
        {
@@ -380,7 +413,7 @@ QString TreeView::findName(KDesktopFile *df, bool deleted)
              KDesktopFile df2(*it);
              name = df2.readName();
 
-             if (!name.isEmpty() && (name != "empty"))
+             if (!name.isEmpty() && (name != QLatin1String("empty")))
                 return name;
           }
        }
@@ -439,6 +472,19 @@ TreeItem *TreeView::createTreeItem(TreeItem *parent, QTreeWidgetItem *after, Men
     return item;
 }
 
+TreeItem *TreeView::createTreeItem(TreeItem *parent, QTreeWidgetItem *after, MenuSeparatorInfo *, bool init)
+{
+    TreeItem* item;
+    if (parent) {
+        item = new TreeItem(parent, after, QString(), init);
+    } else {
+        item = new TreeItem(this, after, QString(), init);
+    }
+
+    setItemWidget(item, 0, new SeparatorWidget);
+    return item;
+}
+
 void TreeView::fillBranch(MenuFolderInfo *folderInfo, TreeItem *parent)
 {
     QString relPath = parent ? parent->directory() : QString();
@@ -458,6 +504,12 @@ void TreeView::fillBranch(MenuFolderInfo *folderInfo, TreeItem *parent)
           after = createTreeItem(parent, after, subFolder);
           continue;
        }
+       MenuSeparatorInfo *separator = dynamic_cast<MenuSeparatorInfo*>(info);
+       if (separator)
+       {
+          after = createTreeItem(parent, after, separator);
+          continue;
+       }
     }
 }
 
@@ -471,7 +523,7 @@ void TreeView::closeAllItems(QTreeWidgetItem *item)
 
 TreeItem *TreeView::expandPath(TreeItem *item, const QString &path)
 {
-   int i = path.indexOf("/");
+   int i = path.indexOf(QStringLiteral("/"));
    QString subMenu = path.left(i+1);
    QString restMenu = path.mid(i+1);
 
@@ -515,7 +567,7 @@ void TreeView::selectMenu(const QString &menu)
        restMenu += '/';
 
    TreeItem *item = 0;
-   int i = restMenu.indexOf("/");
+   int i = restMenu.indexOf(QStringLiteral("/"));
    QString subMenu = restMenu.left(i+1);
    restMenu = restMenu.mid(i+1);
 
@@ -666,7 +718,7 @@ QStringList TreeView::fileList(const QString& rPath)
     QString relativePath = rPath;
 
     // truncate "/.directory"
-    int pos = relativePath.lastIndexOf("/.directory");
+    int pos = relativePath.lastIndexOf(QLatin1String("/.directory"));
     if (pos > 0) relativePath.truncate(pos);
 
     QStringList filelist;
@@ -679,7 +731,7 @@ QStringList TreeView::fileList(const QString& rPath)
         if(!dir.exists()) continue;
 
         dir.setFilter(QDir::Files);
-        dir.setNameFilters(QStringList() << "*.desktop;*.kdelnk");
+        dir.setNameFilters(QStringList() << QStringLiteral("*.desktop;*.kdelnk"));
 
         // build a list of files
         const QStringList files = dir.entryList();
@@ -705,7 +757,7 @@ QStringList TreeView::dirList(const QString& rPath)
     QString relativePath = rPath;
 
     // truncate "/.directory"
-    int pos = relativePath.lastIndexOf("/.directory");
+    int pos = relativePath.lastIndexOf(QLatin1String("/.directory"));
     if (pos > 0) relativePath.truncate(pos);
 
     QStringList dirlist;
@@ -721,7 +773,7 @@ QStringList TreeView::dirList(const QString& rPath)
         // build a list of subdirs
         const QStringList subdirs = dir.entryList();
         for (QStringList::ConstIterator it = subdirs.constBegin(); it != subdirs.constEnd(); ++it) {
-            if ((*it) == "." || (*it) == "..") continue;
+            if ((*it) == QLatin1String(".") || (*it) == QLatin1String("..")) continue;
             // does not work?!
             // if (dirlist.contains(*it)) continue;
 
@@ -778,7 +830,7 @@ static QString createDesktopFile(const QString &file, QString *menuId, QStringLi
    QString base = file.mid(file.lastIndexOf('/')+1);
    base = base.left(base.lastIndexOf('.'));
 
-   QRegExp r("(.*)(?=-\\d+)");
+   QRegExp r(QStringLiteral("(.*)(?=-\\d+)"));
    base = (r.indexIn(base) > -1) ? r.cap(1) : base;
 
    QString result = KService::newServicePath(true, base, menuId, excludeList);
@@ -809,7 +861,7 @@ static QString createDirectoryFile(const QString &file, QStringList *excludeList
       if (i == 1)
          result = base + ".directory";
       else
-         result = base + QString("-%1.directory").arg(i);
+         result = base + QStringLiteral("-%1.directory").arg(i);
 
       if (!excludeList->contains(result))
       {
@@ -855,7 +907,7 @@ bool TreeView::dropMimeData(QTreeWidgetItem *item, int index, const QMimeData *d
         }
     }
 
-    QString folder = parentItem ? parentItem->directory() : "/";
+    QString folder = parentItem ? parentItem->directory() : QStringLiteral("/");
     MenuFolderInfo *parentFolderInfo = parentItem ? parentItem->folderInfo() : m_rootFolder;
     //qDebug() << "think we're dropping on" << (parentItem ? parentItem->text(0) : "Top Level") <<  index;
 
@@ -1026,6 +1078,14 @@ bool TreeView::dropMimeData(QTreeWidgetItem *item, int index, const QMimeData *d
 
         TreeItem *newItem = createTreeItem(parentItem, after, entryInfo);
         setCurrentItem(newItem);
+    } else  {
+        // copying a separator
+        if (action != Qt::CopyAction) {
+            del(dragItem, false);
+        }
+
+        TreeItem *newItem = createTreeItem(parentItem, after, m_separator);
+        setCurrentItem(newItem);
     }
 
     //qDebug() << "setting the layout to be dirty at" << parentItem;
@@ -1099,7 +1159,7 @@ void TreeView::newsubmenu()
    folderInfo->caption = parentFolderInfo->uniqueMenuCaption(caption);
    folderInfo->id = m_menuFile->uniqueMenuName(folder, caption, parentFolderInfo->existingMenuIds());
    folderInfo->directoryFile = file;
-   folderInfo->icon = "package";
+   folderInfo->icon = QLatin1String("package");
    folderInfo->hidden = false;
    folderInfo->setDirty();
 
@@ -1194,6 +1254,35 @@ void TreeView::newitem()
    setLayoutDirty(parentItem);
 }
 
+void TreeView::newsep()
+{
+   TreeItem *parentItem = 0;
+   TreeItem *item = (TreeItem*)selectedItem();
+
+   if(!item)
+   {
+      parentItem = 0;
+   }
+   else if(item->isDirectory())
+   {
+      parentItem = item;
+      item = 0;
+   }
+   else
+   {
+      parentItem = static_cast<TreeItem*>(item->parent());
+   }
+
+   // create the TreeItem
+   if(parentItem)
+      parentItem->setExpanded(true);
+
+   TreeItem *newItem = createTreeItem(parentItem, item, m_separator, true);
+
+   setCurrentItem(newItem);
+   setLayoutDirty(parentItem);
+}
+
 void TreeView::cut()
 {
     copy( true );
@@ -1383,6 +1472,16 @@ void TreeView::paste()
 
       setCurrentItem(newItem);
    }
+   else
+   {
+      // create separator
+      if(parentItem)
+         parentItem->setExpanded(true);
+
+      TreeItem *newItem = createTreeItem(parentItem, item, m_separator, true);
+
+      setCurrentItem(newItem);
+   }
    setLayoutDirty(parentItem);
 }
 
@@ -1433,6 +1532,15 @@ void TreeView::sortItem(TreeItem *item, const SortType& sortType)
     // sort children groups, splited by separator items
     QList<QTreeWidgetItem*>::iterator startIt = children.begin();
     QList<QTreeWidgetItem*>::iterator currentIt = children.begin();
+    while (currentIt != children.end()) {
+        TreeItem *child = static_cast<TreeItem*>(*currentIt);
+        // if it's a separator, sort previous items and continue on following items
+        if (child->isSeparator() && startIt != currentIt) {
+            sortItemChildren(startIt, currentIt, sortType);
+            startIt = currentIt + 1;
+        }
+        ++currentIt;
+    }
     sortItemChildren(startIt, currentIt, sortType);
 
     // insert sorted children in the tree
@@ -1440,6 +1548,10 @@ void TreeView::sortItem(TreeItem *item, const SortType& sortType)
     foreach (QTreeWidgetItem *child, children) {
         // recreate item widget for separators
         TreeItem *treeItem = static_cast<TreeItem*>(child);
+        if (treeItem->isSeparator()) {
+            setItemWidget(treeItem, 0, new SeparatorWidget);
+        }
+
         // try to sort sub-children
         sortItem(static_cast<TreeItem*>(child), sortType);
     }
@@ -1520,6 +1632,14 @@ void TreeView::moveUpOrDownItem(bool isMovingUpAction)
     parentItem->removeChild(sourceItem);
     parentItem->insertChild(destIndex, sourceItem);
 
+    // recreate item widget for separators
+    if (sourceItem->isSeparator()) {
+        setItemWidget(sourceItem, 0, new SeparatorWidget);
+    }
+    if (destItem->isSeparator()) {
+        setItemWidget(destItem, 0, new SeparatorWidget);
+    }
+
     // set the focus on the source item
     setCurrentItem(sourceItem);
 
@@ -1667,17 +1787,17 @@ static QStringList extractLayout(QTreeWidget *tree, QTreeWidgetItem *parent)
         if (item->isDirectory()) {
             if (firstFolder) {
                 firstFolder = false;
-                layout << ":M"; // Add new folders here...
+                layout << QStringLiteral(":M"); // Add new folders here...
             }
             layout << (item->folderInfo()->id);
         } else if (item->isEntry()) {
             if (firstEntry) {
                 firstEntry = false;
-                layout << ":F"; // Add new entries here...
+                layout << QStringLiteral(":F"); // Add new entries here...
             }
             layout << (item->entryInfo()->menuId());
         } else {
-            layout << ":S";
+            layout << QStringLiteral(":S");
         }
     }
 
@@ -1779,7 +1899,7 @@ void TreeView::restoreMenuSystem()
 {
     if ( KMessageBox::warningYesNo( this, i18n( "Do you want to restore the system menu? Warning: This will remove all custom menus." ) )==KMessageBox::No )
         return;
-    QString kmenueditfile = KStandardDirs::locateLocal("xdgconf-menu", "applications-kmenuedit.menu");
+    QString kmenueditfile = KStandardDirs::locateLocal("xdgconf-menu", QStringLiteral("applications-kmenuedit.menu"));
     if ( QFile::exists( kmenueditfile ) )
     {
         if ( !QFile::remove( kmenueditfile ) )
@@ -1796,6 +1916,7 @@ void TreeView::restoreMenuSystem()
     clear();
     cleanupClipboard();
     delete m_rootFolder;
+    delete m_separator;
 
     m_layoutDirty = false;
     m_newMenuIds.clear();
@@ -1803,6 +1924,7 @@ void TreeView::restoreMenuSystem()
     m_menuFile->restoreMenuSystem(kmenueditfile);
 
     m_rootFolder = new MenuFolderInfo;
+    m_separator = new MenuSeparatorInfo;
 
     readMenuFolderInfo();
     fill();
@@ -1817,12 +1939,14 @@ void TreeView::updateTreeView(bool showHidden)
     clear();
     cleanupClipboard();
     delete m_rootFolder;
+    delete m_separator;
 
     m_layoutDirty = false;
     m_newMenuIds.clear();
     m_newDirectoryList.clear();
 
     m_rootFolder = new MenuFolderInfo;
+    m_separator = new MenuSeparatorInfo;
 
     readMenuFolderInfo();
     fill();
@@ -1833,7 +1957,7 @@ void TreeView::updateTreeView(bool showHidden)
 
 void TreeView::sendReloadMenu()
 {
-    QDBusMessage message = QDBusMessage::createSignal("/kickoff", "org.kde.plasma", "reloadMenu");
+    QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/kickoff"), QStringLiteral("org.kde.plasma"), QStringLiteral("reloadMenu"));
     QDBusConnection::sessionBus().send(message);
 }
 
